# Accounts.pm - The Object Class that provides a Accounts Object
#$Id: Accounts.pm,v 1.26 2006/04/10 16:03:11 moreejt Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.

package Portal::Accounting::Accounts;
use strict;
use Portal::Accounting::Base;
use Portal::Accounting::Objects::Account;
use Portal::Accounting::Objects::ChartOfAccounts;
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Portal::Accounting::Base Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '$Revision: 1.26 $';

# global variables.

# modify to represent the commands this state provides.
my %commands = (
  display => "Chart of Accounts",
  new => "New Account",
  save => "Save Account",
  edit => "Edit Account",
  delete => "Delete Account",
  assign => "Assign Accounts",
  javascript => 'javascript test',
);

=head1 NAME

Accounts - Object used to work with accounts

=head1 SYNOPSIS

  use Portal::Accounting::Accounts;
  my $obj = Portal::Accounting::Accounts->new;
  if ($obj->error())
  {
    die $obj->errorMessage();
  }
  $obj->run(command => $command);

=head1 DESCRIPTION

Accounts is the Accounts Menu from which every other part of
the systems will be accessed.

=head1 Exported FUNCTIONS

B<NOTE>: I<bool> = 1(true), 0(false)

=over 4

=item scalar new(codeVersion, release, DBVersion)

 Creates a new instance of the Portal::Accounting::Accounts module.
 See Portal::Accounting::Base(3) for a listing of required arguments.

 Instantiates an instance of the Accounts State. !!Anything passed in here should be
 data that never changes from one run of a command to another such as database handles
 portalSession? input? cookieObj? dont fit that criterion

=cut

sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  my %args = ( @_ );

  if ($self->error)
  {
    $self->prefixError();
    return $self;
  }

  # instantiate anything unique to this module
  $self->{commands} = \%commands;
  $self->{object} = "Account";

  # do validation
  if (!$self->Portal::Accounting::Accounts::isValid)
  {
    # the error is set in the isValid() method.
    return $self;
  }

  # do anything else you might need to do.
  my $l = $self->{langObj};
  $self->{metaTypes} = { 1 => $l->map("Asset"),
    2 => $l->map("Liability"),
    3 => $l->map("Equity"),
    4 => $l->map("Revenue"),
    5 => $l->map("Expense"),
  };
  return $self;
}

=item % getCommands(void)

  This method returns the commands hash and is only to be
  used by the registerApp method for creating the permissions,
  if using the AppStateCommandSecurity implementation.

=cut
sub getCommands
{
  return %commands;
}

=item bool isValid(void)

 Returns 0 or 1 to indicate if the object is valid.
 The error will be available via errorMessage().

=cut

sub isValid
{
  my $self = shift;

  # make sure our Parent class is valid.
  if (!$self->SUPER::isValid())
  {
    $self->prefixError();
    return 0;
  }

  # validate our parameters.

  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->error($self->genErrorString("all"));
    return 0;
  }

  return 1;
}

=item HTMLObject c_display()

=cut
sub c_display
{
  my $self = shift;
  my %args = ( command => "display", @_ );
  my $command = $args{command};
  my $doc = $self->prepDoc;
  my $lc = "";
  if (defined $self->input->{ledgerCode}) { $lc = $self->input->{ledgerCode}; }
  if (defined $self->input->{message}) { $doc->print($self->input->{message}); }

  my $chart = Portal::Accounting::Objects::ChartOfAccounts->new(dbHandle => $self->{dbHandle}, langObj => $self->{langObj}, ledgerCode => $lc);
  if ($chart->error) { $doc->print($chart->errorMessage); return $doc;  }
  $doc->print($chart->printHTML);
  if ($chart->error) { $doc->print($chart->errorMessage); }
  return $doc;
}

=item HTMLObject c_edit()

  this routine is to create/edit a new account

=cut
sub c_edit
{
  my $self = shift;
  my ($type, $name, $code, $embed);
  if (defined $self->{input}->{type}) { $type = $self->{input}->{type}; }
  if (defined $self->{input}->{name}) { $name = $self->{input}->{name}; }
  if (defined $self->{input}->{code}) { $code = $self->{input}->{code}; }
  if (defined $self->{input}->{embed}) { $embed = $self->{input}->{embed}; }
  $self->{OBJECT} = Portal::Accounting::Objects::Account->new(
    langObj => $self->langObj,
    dbHandle => $self->dbHandle,
    meta => $self->{input}->{meta},
    ledgerCode => $self->{input}->{ledgerCode},
    type => $type,
    name => $name,
    code => $code,
    embed => $embed,
  );
  my $doc = $self->SUPER::c_edit(@_);
  my $account = $self->{OBJECT};
  my $command = $self->{input}->{command};
  my $lc = "";
  if (defined $self->{input}->{ledgerCode}) { $lc = $self->{input}->{ledgerCode}; }
  if ($self->error) { $doc->print($self->errorMessage); $self->resetError; } #!!! dont want to do it this way but if $self->error is set portal craps out even though a doc is returned!!!!
#  if ($account->error) { $self->error($account->errorMessage); return $doc; }
  elsif ($command eq "save" && $lc ne '' && $account->isNew) #in a ledger, may be embedded.
  {
    my $id = $account->insertId;
    #assign to current ledger
    $account->assignAccounts(ledgerCode => $lc, accounts => [ $id ] );
    if ($account->error) { $doc->print($account->errorMessage); }
    $doc->setFocus("body");
    if (defined $self->{input}->{embed} && $self->{input}->{embed} == 1)
    {
    #!! need js code in onload to write to parent window
      my $aName = $account->get("name");
      my $aCode = $account->get("code");
      my $aId = $id;
      my $aTaxable = $account->get("taxable");
      if ($aTaxable == 1) { $aTaxable = 'true'; }
      else { $aTaxable = 'false'; }
      my $aMeta = $account->get("meta");
      my $aType = $account->get("type");
      if ($aTaxable == 1) { $aName .= "*"; }
      $doc->setOnload( <<"ENDOF" );
  var account = new Object();
  account.name = '$aName';
  account.code = '$aCode';
  account.id = '$aId';
  account.taxable = $aTaxable;
  account.type = '$aType';
  account.meta = '$aMeta';
  account.active = true;

  if (opener)
  {
    if (opener.afterNewAccount) opener.afterNewAccount(account);
    else { alert("function afterNewAccount not found"); }
  }
ENDOF
      $doc->print(<<"ENDOF");
  <form name="main">
    <input type="hidden" name="name" value="$aName">
    <input type="hidden" name="code" value="$aCode">
    <input type="hidden" name="id" value="$aId">
    <input type="hidden" name="taxable" value="$aTaxable">
    <input type="hidden" name="meta" value="$aMeta">
    <input type="hidden" name="type" value="$aType">
  </form>
ENDOF
      return $doc;  #return bc embedded does not get main button
    }
  }
  if (!defined $self->{input}->{embed} || $self->{input}->{embed} == 0)
  {
    my $ChartOfAccounts = $self->langObj->map("Chart of Accounts");
    $doc->print(<<"ENDOF");
  <form name="display">
    <input type="submit" value="$ChartOfAccounts">
    <input type="hidden" name="app" value="Accounting">
    <input type="hidden" name="state" value="Accounts">
    <input type="hidden" name="command" value="display">
    <input type="hidden" name="ledgerCode" value="$lc">
  </form>
ENDOF
  }
  return $doc;
}

sub c_assign
{
  my $self = shift;
  my $doc = $self->prepDoc();
  my $account = Portal::Accounting::Objects::Account->new(langObj => $self->{langObj}, dbHandle => $self->{dbHandle}, meta => $self->{input}->{meta}, ledgerCode => $self->{input}->{ledgerCode});
  if ($account->error) { $doc->print($account->errorMessage); return $doc;  }
  #walk assign unassign vars and build up arrays
  my $assign = $self->{input}->{assign};
  my $unassign = $self->{input}->{unAssign};
  $assign =~ s/:$//;
  $unassign =~ s/:$//;
  my @assign = split(":", $assign);
  my @unassign = split(":", $unassign);
  #call assign
  if (@assign > 0)
  {
    $assign = $account->assignAccounts(ledgerCode => $self->{input}->{ledgerCode}, accounts => \@assign );
  }
  #call unassign
  if (@unassign > 0)
  {
    $unassign = $account->unAssignAccounts(ledgerCode => $self->{input}->{ledgerCode}, accounts => \@unassign );
  }
  if ($account->error) { $doc->print($account->errorMessage);  }
  #output # records and link back to chart of accounts
  my $r = $self->{langObj}->map("records modified");
  my $chart = $self->{langObj}->map("Chart of Accounts");
  $doc->print($assign + $unassign . " $r.<br>\n");
  $doc->print(<<"ENDOF");
  <form name="display">
    <input type="hidden" name="app" value="$self->{app}">
    <input type="hidden" name="state" value="$self->{state}">
    <input type="hidden" name="command" value="display">
    <input type="hidden" name="ledgerCode" value="$self->{input}->{ledgerCode}">
    <input type="submit" value="$chart">
  </form>
ENDOF
  $doc->setOnload('return document.display.submit();');
  return $doc;
}

sub prepDoc
{
  my $self = shift;
  my $doc = $self->SUPER::prepDoc(@_);
  $doc->setJavascriptInclude(file => $self->{jsDir}."Accounts.js");
  return $doc;
}

sub c_javascript
{
  my $self = shift;
  my $doc = HTMLObject::Normal->new;
  $doc->setContentType('text/plain');
  my $a = Portal::Accounting::Objects::Account->new(langObj => $self->langObj, dbHandle => $self->dbHandle);
  $a->set('ledgerCode', $self->{input}->{ledgerCode});
  if ($a->error) { $self->error($a->errorMessage); }
  else
  {
    $doc->print($a->javascriptOverhead);
  }
  if ($a->error) { $self->error($a->errorMessage); }
  if (!$self->error) { return $doc; }
  return undef;
}

=back

=cut

1;
__END__

=head1 NOTE

 All data fields are accessible by specifying the object
 and pointing to the data member to be modified on the
 left-hand side of the assignment.
 Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

Xperience, Inc. (mailto:xiwa@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal(3), Portal::Accounting(3), Portal::AppState(3)

=cut
