# Investment.pm - The Object Class that provides a Investment Object
#$Id: Event.pm,v 1.28.4.3 2007/03/26 03:21:20 moreejt Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2003 HLR http://www.pcxperience.org  All rights reserved.

package Portal::Accounting::Investments::Objects::Event;
use strict;
use Date::Calc qw(Delta_Days);
use Portal::Accounting::Investments::Objects::Base;
use Portal::Accounting::Investments::Objects::LotNode;
use Portal::Accounting::Objects::JournalEntry;
use Portal::Accounting::Objects::Account;
use Portal::Accounting::Investments::Objects::Stock;
use Portal::Accounting::Objects::SubEntity;

use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Portal::Accounting::Investments::Objects::Base Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '$Revision: 1.28.4.3 $';

=head1 NAME

Accounting::Investments::Objects::Event

=head1 SYNOPSIS

  use Portal::Accounting::Investments::Objects::Event;
  @ISA = qw(Portal::Accounting::Investments::Objects::Event Exporter AutoLoader);

=head1 DESCRIPTION

 Event - Object for deriving investment event objects such as Buy, Sell, etc.

  #################NOTES#########
  #
  #  the user defined counter feature is currently not implemented
  #  1) front end code does not support it yet
  #  2) back end code is not complete
  #  3) lot balances become tied not to dates but to individual events
  #  4) retrieving lot balances is difficult enough already
  #  5) database tables are a mess--need rewrite
  #  6) in sec_event_tb, date + counter is primary_key and event_ref can be same because some events have two records --each having different counter
  #################################################

=head1 Exported FUNCTIONS

B<NOTE>: I<bool> = 1(true), 0(false)

=over 4

=item scalar new(dbHandle => , langObj =>, ledgerCode =>)

    Creates a new instance of Portal::Accounting::Investments::Objects::Event.
    See Portal::Objects::Base(3) for a listing of required arguments if the
    above args don't seem to work.

=cut
sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  my %args = (ledgerCode => '', @_);
  $self->{table} = "sec_event_tb";
  $self->{id} = "id";
  $self->{column} = 'event_ref';

  if ($self->error)
  {
    return $self;
  }

  # instantiate anything unique to this module
  $self->{data}->{main} = {
    "id" => { -Type => "hidden", },
    "ledgerCode" => { -Type => "hidden", -Value => $args{ledgerCode}, },
    "counter" => { -Type => "hidden", },
    "date" => { -Type => "hidden", },
  };
  # do anything else you might need to do.
  $self->{INSERTorder}[0] = [ qw( event_code  buy_date  trans_date  counter  sub_e_code  a  s  l  os  ns  cc  ledger_code  event_ref  comment ) ] ;  #insert order for db column names
  $self->{SELECT}[0] = "SELECT ##COLUMNS## FROM $self->{table} WHERE $self->{column} = #FIELD=id#";
  $self->{INSERT}[0] = "INSERT INTO $self->{table} (##COLUMNS##) VALUES (##VALUES##)";
  $self->{UPDATE}[0] = "UPDATE $self->{table} SET ##UPDATE## WHERE $self->{column} = #FIELD=id#";
  $self->{DELETE}[0] = "DELETE FROM $self->{table} WHERE $self->{column} = #FIELD=id#";
  $self->{deleteString} = <<"ENDOF";
     <td class="delete">
      <div onclick="if (confirm('Delete #VALUE=id#. Are you sure?')) { document.getElementById('i#VALUE=id#').addClass('deleting'); deleteInvst('#VALUE=id#', '#VALUE=ledgerCode#'); }">X</div>
   </td>
ENDOF
  $self->{ledger} = Portal::Accounting::Objects::Ledger->new(langObj => $self->langObj, dbHandle => $self->dbHandle);
  $self->{ledger}->set('code', $args{ledgerCode});
  $self->{ledger}->select;
  if ($self->{ledger}->error) { $self->error($self->{ledger}->errorMessage); }
  return $self;
}

=item bool isValid(void)

 Returns 0 for invalid or 1 for valid.
 The error will be available via errorMessage().

=cut
sub isValid
{
  my $self = shift;

  my $d = $self->get('date');
  if ($d =~ /^(\d{4})-(\d{1,2})-(\d{1,2})$/) { $self->set('date', "$1-".sprintf("%02d",$2)."-".sprintf("%02d",$3) ) }
  # make sure our Parent class is valid.
  if (!$self->SUPER::isValid())
  {
    $self->error('');
    return 0;
  }

  # validate our parameters.
  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->error($self->genErrorString("all"));
    return 0;
  }

  return 1;
}

=item nextLotNumber

 looks up the next number from the new_lot_number_seq sequence in the database
 returns undef on failure or the new lot number on success

=cut
sub nextLotNumber
{
  my $self = shift;
  my $sth = $self->{dbHandle}->read(sql => "SELECT nextval('new_lot_number_seq')");
  if ($self->{dbHandle}->error())
  {
    $self->error("<p>Error retrieving new lot number!<p>" . $self->{dbHandle}->errorMessage);
    return undef;
  }
  if (my @info = $sth->fetchrow_array) { return $info[0]; }
  $self->error("<p>No lot returned in nextLotNumber</p>");
  return undef;
}

=item integer nextSecurityEventCounter(date)

  gets the next available counter for the given date.

=cut
sub nextSecurityEventCounter
{
  my $self = shift;
  my $date = shift;
  my $sth = $self->{dbHandle}->read(
    sql => "SELECT next_sec_event_tb_counter(?)",
    plug => [ $date ]
  );
  if ($self->dbHandle->error) { $self->error($self->dbHandle->errorMessage); return undef; }
  my $counter = 0;
  if (my @data = $sth->fetchrow_array) { $counter = $data[0]; }
  return $counter;
}

=item integer nextSecurityEventRef()

  summary:  returns next event ref serial for use in sec_event_tb

=cut
sub nextSecurityEventRef
{
  my $self = shift;
  my $sth = $self->{dbHandle}->read(sql => "SELECT nextval('sec_event_tb_event_ref_seq')");
  if ($self->{dbHandle}->error())
  {
    $self->error("<p>Failed to get the nextval() of the security event reference number sequence!</p>" . $self->{dbHandle}->errorMessage);
    return -1;
  }
  my @info = $sth->fetchrow_array;
  return $info[0];
}

=item boolean propagateLot(Investments::Objects::LotNode)

 requires:  LotNode
 summary: this routine will create the next nodes on the lot's tree if there are events in the future.
   this function recursively calls itself until there are no more
   nodes/events to work on.

=cut
sub propagateLot
{
  my $self = shift;
  my $newNode = shift;
  if ($newNode->get('shares') <= 0 ) { return 1; }
  my $lc = $self->get('ledgerCode');
  #have data internal to this object if called from a subclassed objects as $self->propagateLot

  #for given lot node, get next event that affects it
  #propagate event into lot_node_tb as entry.  this will make different types of entries based on the event
  # also make sure that the dates for the events gathered are greater than or equal to the beginning date of the ledger for the lots
  #need global events for this sub account
  # and local events for this lot

#old sql code is hard to maintain and doesn't seem to work
#   SELECT event_ref
#   FROM sec_event_tb
#   WHERE
#     event_code <> 'memo'
#     AND event_code <> 'buy'
#     AND (ledger_code = '*' OR ledger_code = ?)
#     AND ((trans_date = ? AND counter > ?)  OR  trans_date > ?)
#     AND (S = ? OR (event_code = 'acqs' AND A = ? ))
#     AND (L IS NULL or L = ?)
#   LIMIT 1

  $self->{dbHandle}->write("SET DATESTYLE='ISO'");
  my $sql = <<"ENDOF";
  SELECT event_ref, trans_date, counter
  FROM sec_event_tb
  WHERE
    ( (trans_date = ? AND counter > ?) OR  trans_date > ? )
    AND (
      (
        ledger_code = '$lc'
        AND L = ?
        AND ( event_code = 'sell' OR event_code = 'sxfr' OR event_code = 'adjs' )
      )
      OR (
        S = ?
        AND (event_code = 'spof' OR event_code = 'splt' OR event_code = 'rtcp' OR event_code = 'acqs')
      )
    )
  ORDER BY trans_date, counter LIMIT 1
ENDOF
  my $plug = [
    $newNode->get('date'),
    $self->get('counter'),
    $newNode->get('date'),
    $newNode->get('lot'),
    $newNode->get('subAccount'),
  ];
#  $self->log($sql.":".join(',',@{$plug}));
#if ($self->get('type') eq 'sell' && $newNode->get('id') > 5450) {$self->error('INFINITE LIMIT:'.$sql.":".join(',',@{$plug})); return undef;}
  my $sth = $self->{dbHandle}->read(sql => $sql, plug => $plug );
  if ($self->dbHandle->error) { $self->error($self->dbHandle->errorMessage); return undef; }
  if (my @data = $sth->fetchrow_array)
  {
    my $event = $self->getEvent($data[0]);
    if (!defined $event)
    {  $self->error("<p>$sql:".join(",",@{$plug})."</p>"); return 0; }
    elsif ($event->error)
    {  $self->error($event->errorMessage); return 0; }
    my $r = $event->propagate($newNode);  #propagate is also updating database
    if ($event->error) { $self->error($event->errorMessage); return $r; }
  }
  return $newNode;
}

=item boolean propagate(LotNode)

  This routine only propagates the given node and performs any other common
  event propagation actions.  It is mean to be called from subclassed events.

  ALL EVENTS WILL USE THIS GENERAL METHOD FOR propagate
  get affected nodes took place in insert and propagate is called on each one
  propagate Delete is done first on original node (also done during insert)

  calculate new lot node data from old lot and data from this event
  check to make sure new data is valid
  build and insert a JournalEntry if needed
  build a new lot node with new data (incl. JournalEntry if needed)
  insert new node or nodes
  foreach new node self->SUPER::propagate(node)


=cut
sub propagate
{
  my $self = shift;
  return $self->propagateLot(@_)
}

=item boolean deleteEvents(void)

  Call this routine to delete an investment event.  It will handle
  dependent events that will become invalid and delete them also.
  It is mean to be called from subclassed objects.

=cut
sub deleteEvents
{
  my $self = shift;
  my $db = $self->dbHandle;
  my %parents;
  my @events;
  #get node or nodes that goes with current event.  perhaps in other ledgers too?
  my $sql = "SELECT node_id FROM lot_node_tb WHERE event_ref = ?";
  $self->log("delete: ".$sql." ".$self->get('id'));
  my $sth = $db->read(sql => $sql, plug => [$self->get('id')]);
  while (my @data = $sth->fetchrow_array)
  {
    my $node = Portal::Accounting::Investments::Objects::LotNode->new(langObj => $self->{langObj}, dbHandle => $db, ledgerCode => $self->get('ledgerCode'));
    $node->set('id', $data[0]);
    $node->select;
#    $self->log('parent of node '.$node->get('id').' is '.$node->get('parent'). 'and type is '.$node->get('type'));
    if ($node->error) { $self->error($node->errorMessage); }
    else
    {
      my $p = $node->get('parent');
      if ($p ne '') { $parents{$p} = undef; }
      #this function should walk the entire tree including new trees created from spinoff, acqs, transfers etc.
      #this will include this event (self) if it is local
      @events = $node->findLocalEventsInTree();
      $self->log('findLocalEventsInTree for node '.$node->get('id'). ' yeiled '.(scalar @events).' entries');
      if ($node->error) { $self->error($node->errorMessage); }
    }
    #delete node tree first bc nodes use sec events
    if (!$self->error)
    {
      $node->deleteTree;
      if ($node->error) { $self->error($node->errorMessage); }
      #delete all local events
      if (scalar @events > 0)
      {
        $sql = "DELETE FROM sec_event_tb WHERE event_ref IN (".join(',', @events).")";
        $self->log($sql);
        $db->write(sql => $sql);
        if ($db->error) { $self->error($db->errorMessage); }
      }
    }
  }

  $self->SUPER::delete;
  if ($self->error) { return undef; }
  foreach my $parent (keys %parents)
  {
    $parents{$parent} = Portal::Accounting::Investments::Objects::LotNode->new(langObj => $self->{langObj}, dbHandle => $db, ledgerCode => $self->get('ledgerCode'));
    $parents{$parent}->set('id',$parent);
    $parents{$parent}->select;
    #the parent node needs to be propogated using the event that created it?
    if ($parents{$parent}->error) { $self->error($parents{$parent}->errorMessage); }
    else { $parents{$parent}->{EVENT}->propagate($parents{$parent}); }
  }

  if (!$self->error) { $self->dbHandle->commit; }
  else { $self->dbHandle->rollback; return undef; }
  return @events;
}

=item Investments::Objects::Event getEvent(id)

  finds the event type for the given event id and returns that type of object with the event data.  The type will be Buy, Sell, etc.

=cut
sub getEvent
{
  my $self = shift;
  my $id = shift;
  my $lc = $self->get('ledgerCode');
  if ($lc eq '*') { $self->error("<div> Cannot use * as ledgerCode in getEvent.  propagation gets all screwed up.</div>"); return undef; }
  my $type = $self->getEventType($id);
  if ($self->error) { return undef; }
  my $module = $self->{objects}->{$type};
  eval("use Portal::Accounting::Investments::Objects::$module;");
  my $object;
  eval('$object = Portal::Accounting::Investments::Objects::'.$module.'->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $lc);');
  if (!defined $object) { $self->error("<p>Object is undefined when trying to create module '$module' for id '$id'.</p>"); return undef; }
  if ($object->error) { $self->error($object->errorMessage); }
  $object->set('id',$id);
  $object->select;  #ledgercode here is being overwritten with the db data.  if * that is causing a problem.
  if ($object->error) { $self->error($object->errorMessage); }
  $object->set('ledgerCode', $lc);
  return $object;
}

=item string getEventType(event)

  requires:  event - integer

  summary: this returns a string representing the type of security event for the given event id
  Valid return values (type) are:
    type => Object Name       #old return value -- now returns type
    ------------------------------------------
    buy  => "Buy",            #10
    sell => "Sell",           #20
    adjs  => "Adjustment",     #40 used to be fee
    memo => "Memo",           #memo
    acqs => "Merger",         #80
    rtcp => "ReturnOfCapital", #50
    spof => "SpinOff",        #70, #60 was stock swap. deprecated.
    splt => "StockSplit",     #90
    sdiv => "StockDividend",  #100  #deprecated!!! will remove in the future
    sxfr => "StockTransfer"   #30

=cut
sub getEventType
{
  my $self = shift;
  my $id = shift;
  my $sql = "SELECT event_code FROM sec_event_tb WHERE event_ref = ? LIMIT 1";
  my $sth = $self->{dbHandle}->read(sql => $sql, plug => [ $id ]);
  if (my @data = $sth->fetchrow_array) { return $data[0]; }
  $self->error('<div class="error">No valid event type found in getEventType for id "'.$id.'"!  Perhaps this event was already deleted.</div>');
  return undef;
}

=item boolean insert

  You should probably NOT call this function directly.  You should look at insertEvent.

  This routine will call the parent insert method after checking the
  counter and id fields.  If neccessary a new id and counter will be
  selected from the database.  meant to be called from subclassed objects.
  It only inserts a record for the current object and not dependent objects.

=cut
sub insert
{
  my $self = shift;
  if (!$self->dateCheck) { $self->error('<div class="error">Date cannot be before last close out of '.$self->{ledger}->lastClose.'</div>'); return undef; }
  if ($self->get('id') eq '')
  {
    $self->set('id', $self->nextSecurityEventRef);
    $self->set('counter', $self->nextSecurityEventCounter);
    $self->log("setting id ".$self->get('id')." and counter ".$self->get('counter'));
  }
  if (!$self->error) { return $self->SUPER::insert(@_); }
  return undef;
}

=item boolean insertEvent(LotNode)

  summary: Provides common actions for inserting security events to create nodes in lot trees.  It is called from subclassed objects.  It will call propagate(Delete) functions on the given node which selects other events if neccessary and calls propagate on them recursively.

=cut
sub insertEvent
{
  my $self = shift;
  my $node = shift;

  #get event date and compare to ledger closeout
  if (!defined $node) { $self->error('<div class="error">Error! Undefined node sent to insert!</div>'); return 0; }
  if ($node->get('id') == -1) { $self->error('<div class="error">Error! Invalid node sent to insert with id of -1!</div>'); return 0; }
  #after event is inserted the nodes must be updated.
  $self->log("insertEvent: ".$self->get('id').", ".$self->get('counter').":node ".$node->get('id').":".$node->get('ledgerCode'));
  ### insert from sell sent an old node which does have a tree.  so does adjs
  $node->deleteSubTree;
  if ($node->error) { $self->error($node->errorMessage); return 0; }

  # propagate the node into the future
  $self->propagate($node);
  if ($self->error)
  {
    $self->error("<div>Error propagating node (" . $node->get('id') . ") into the future!</div>");
    return 0;
  }

  return 1;
}

=item checkOtherLedgers(date =>, counter =>, stock =>)

  This sub can be called from the security events to handle the inserting of
  global events into other ledgers.  The local ledger insert should be called
  directly.

=cut
sub insertIntoLedgers
{
  my $self = shift;
  my %args = (date => '', counter => '', stock => '', @_);
  my $lobj = Portal::Accounting::Objects::Ledger->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->get('ledgerCode'));
  my @ledgers = $lobj->selectCodesArray;
  if ($lobj->error) { $self->error($lobj->errorMessage); }
  else
  {
    foreach my $ledger (@ledgers)
    {
      $self->log('checking for event '.$self->get('id').' in ledger '.$ledger);
#      if ($ledger == $self->get('ledgerCode')) { next; }
      my $node = Portal::Accounting::Investments::Objects::LotNode->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $ledger);
      my @list = $node->list(date => $args{date}, counter => $args{counter}, stock => $args{stock});
      if ($node->error) { $self->error($node->errorMessage); }
      else
      {
        foreach my $n (@list)
        {
          #swap out ledger?
#          $self->set('ledgerCode', $ledger);
          $self->insertEvent($n);
          if ($self->error) { last; }
        }
      }
    }
  }
}

=item @integer shortTermlongTerm(date, buydate)

  Requires: two dates without labels (dont do date => )
  Returns; Array of integers representing account numbers
  Summary: Takes the sell and buy dates to figure out which accounts to use for capital gains/losses.  Short Term accounts will be returned if 1 year or less is in between the dates.  The capital Gains account will always be first.

=cut
sub shortTermLongTerm
{
  my $self = shift;
  my $sell = shift;
  my $buy = shift;
  my $days = Delta_Days(split("-",$buy) , split("-",$sell) );
  if ($days < 365) { return ($self->{shortTermCG}, $self->{shortTermCL}); }
  return ($self->{longTermCG}, $self->{longTermCL});
}

=item %strings printHeaders

  This set of headers is to be used on the Report forms.  They are separate so that each object can only output the data.  The headers are just at the top of the report insstead of on each line.

=cut
sub printHeaders
{
  my $self = shift;
  my $Event = $self->{langObj}->map('Event');
  my $Detail = $self->{langObj}->map('Detail');
  my $Comment = $self->{langObj}->map('Comment');
  my $Total = $self->{langObj}->map('Total');
  my $Shares = $self->{langObj}->map('Shares');
  my $Price = $self->{langObj}->map('Price');
  my $Date = $self->{langObj}->map('Date');
  my $Lot = $self->{langObj}->map('Lot');
  my $SubEntity = $self->{langObj}->map('Sub Entity');
  my %r;
  $r{body} = <<"ENDOF";
<table class="report tertiary">
  <thead><tr>
    <th class="date">$Date</th>
    <th class="event">$Event</th>
    <th class="detail">$Detail</th>
    <th class="price">$Price</th>
    <th class="shares">$Shares</th>
    <th class="cost">$Total</a></th>
    <th class="subEntity"><a title="$SubEntity">S</a></th>
    <th class="lot">$Lot</th>
    <th class="delete">X</th>
  </tr></thead>
</table>
ENDOF
  $r{css} = <<"ENDOF";
  .report { width: 100%; }
  .date {text-align: left; width: 7em; }
  .event {text-align: left; width: 4em; }
  .lot {text-align: right; width: 4em; }
  .detail {text-align: left;}   /* comment */
  .subEntity {text-align: center; width: 3em; }
  .shares {text-align: right; width: 7em; }
  .cost {text-align: right; width: 10em; }
  .price {text-align: right; width: 7em; }
  .delete {text-align: center; width: 2em; font-size: small; color: red; }
ENDOF
  return %r;
}

sub dateCheck
{
  my $self = shift;
  my $lastClose = $self->{ledger}->lastClose;
  if ($lastClose ne '' && $self->get('date') <= $lastClose)
  {
    return 0;
  }
  return 1;
}

=back

=cut

1;
__END__

=head1 AUTHOR

Xperience, Inc. (mailto:xiwa@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal(3), Portal::Accounting(3), Portal::Objects::Base(3)

=cut