# Stock.pm - The Object Class that provides a Stock Object
# Created by James Pattie & Jason A. Pattie, 11/07/2000.
#$Id: Stock.pm,v 1.8 2006/05/12 22:09:53 moreejt Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.

package Portal::Accounting::Investments::Objects::Stock;
use strict;
use Portal::Accounting::Objects::SubAccount;
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Portal::Accounting::Objects::SubAccount Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '$Revision: 1.8 $';

=head1 NAME

Stock - Object used to build a Stock Object Class.

=head1 SYNOPSIS

  use Portal::Accounting::Investments::Objects::Stock;
  my $obj = Portal::Accounting::Investments::Objects::Stock->new;
  if ($obj->error())
  {
    die $obj->errorMessage();
  }

=head1 DESCRIPTION

Stock is a Stock class.

=head1 Exported FUNCTIONS

B<NOTE>: I<bool> = 1(true), 0(false)

=over 4

=item scalar new(ticker, name, number, type, market)

 Creates a new instance of the Portal::Accounting::Objects::Stock module.
 See Portal::Accounting::Objects::Base(3) for a listing of required arguments.

 requires:
  ticker
  name
  number  - The SubAccount number this stock is associated with
  type
  market

=cut

sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  my %args = ( @_ );

  if ($self->error)
  {
    $self->prefixError();
    return $self;
  }

  # instantiate anything unique to this module
  $self->{name} = "Portal::Accounting::Investments::Objects::Stock";
  my $l = $self->{langObj};
  $self->{markets} = [ qw(None Auto NYSE Nasdaq Amex) ] ;
  my @markets;
  foreach my $m (@{$self->{markets}})
  {
    push @markets, $l->map($m);
  }
  $self->{data}->{main}->{market} = { -Label => $l->map("Market"), -Type => "select", -Options => {values => $self->{markets}, names => \@markets }, -DataType => "int" };
  $self->{data}->{main}->{ticker} = { -Label => $l->map("Ticker"), -Type => "text", size => 7, -DataType => "string" };
  $self->{data}->{main}->{company} = { -Label => $l->map("Company"), -Type => "text", size => 20, -DataType => "string" };
  $self->{data}->{main}->{type} = { -Label => $l->map("Type"), -Value => 70, -Type => "hidden"};
  $self->{data}->{main}->{typeName} = { -Label => $l->map("Stock"), -Type => "text" };

  # do anything else you might need to do.
  my $Submit = $l->map("Save");
  my $title = $l->map("Public/Private Stocks");
  $self->{template}->{main} = <<"ENDOF";
<form #FORMARGS#>
  #FORMERRORSTRING#
  <h4>$title</h4>
  #FIELD=app#
  #FIELD=state#
  #FIELD=command#
  #FIELD=id#
  <table class="primary">
    <tr>
      <td>#LABEL=name#</td><td>#FIR=name#</td>
    </tr>
    <tr>
      <td>#LABEL=code#</td><td>#FIR=code#</td>
    </tr>
    <tr>
      <td>#LABEL=type#</td><td>#LABEL=typeName# #FI=type#</td>
    </tr>
    <tr>
      <td>#LABEL=company#</td><td>#FIR=company#</td>
    </tr>
    <tr>
      <td>#LABEL=ticker#</td><td>#FIR=ticker#</td>
    </tr>
    <tr>
      <td>#LABEL=market#</td><td>#FIR=market#</td>
    </tr>
  </table>
  <input type="submit" name="Submit" value="$Submit">
</form>
#FORMREQUIREDSTRING#
ENDOF
  $self->{javascript}->{main} = <<"ENDOF";
ENDOF
  $self->{profile}->{main} = {
    required => [ qw( name code type ) ],
    optional => [ qw( company ticker id ) ],
    constraints => {
      id => qr/^\d\d+$/,
      type => qr/^\d+$/,
      name => qr/^[\d\w][\d\s\w&.,-]+$/,
      code => qr/^[\d\w.-]+$/,
      company => qr/[^"]/,
      ticker => qr/^[\d\w.-]+$/,
    },
  };
  $self->{defaultForm} = "main";
  ##db infrastructure
  $self->{INSERTorder}[@{$self->{INSERTorder}}] = [ qw( name market ticker) ] ;  #insert order for db column names
  $self->{newOnly}[@{$self->{newOnly}}] = { sub_acct => "id" };
  $self->{db2data}[@{$self->{db2data}}] = { name => "company", sub_acct => 'id' };  #map database columns to ->data hash, not specified assumes same name
  my $index = (scalar @{$self->{data2db}}) - 1;
  $self->{data2db}[$index] = {reverse %{$self->{db2data}[$index]}};

  $self->{SELECT}[@{$self->{SELECT}}] = "SELECT ##COLUMNS## FROM stocks_tb WHERE stocks_tb.sub_acct = #FIELD=$self->{id}#";
  $self->{INSERT}[@{$self->{INSERT}}] = "INSERT INTO stocks_tb (sub_acct, ##COLUMNS##) VALUES ((SELECT sub_acc_number FROM sub_acc_tb WHERE sub_acc_name = '#FIELD=name#' AND code = '#FIELD=code#'), ##VALUES##)";
  $self->{UPDATE}[@{$self->{UPDATE}}] = "UPDATE stocks_tb SET ##UPDATE## WHERE sub_acct = '#FIELD=$self->{id}#'";
  #remember to use single quotes around ##foo## if needed
  $self->{DELETE}[@{$self->{DELETE}}] = "DELETE FROM stocks_tb WHERE sub_acct = #FIELD=$self->{id}#";  #is 'id' a hardcoded form element?
  #find out ifis this being referenced from other tables before deleting
  my $cannot = $l->map("Cannot Delete") ." ". $l->map("Stocks");
  $self->{preDELETE}[@{$self->{preDELETE}}] = "SELECT '$cannot'";
  #!!! use outer join here to get rid of union
  $self->{LIST}[0] = <<"FOO";
  SELECT
    name||' ('||ticker||')' AS name,
    sub_acct AS id
   FROM stocks_tb
   WHERE ticker IS NOT NULL
  UNION
  SELECT
    name,
    sub_acct AS id
   FROM stocks_tb
   WHERE ticker IS NULL
  ORDER BY name
FOO

  return $self;
}

=item bool isValid(void)

 Returns 0 or 1 to indicate if the object is valid.
 The error will be available via errorMessage().

=cut

sub isValid
{
  my $self = shift;

  # make sure our Parent class is valid.
  if (!$self->SUPER::isValid())
  {
    $self->prefixError();
    return 0;
  }

  # validate our parameters.
  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->error($self->genErrorString("all"));
    return 0;
  }

  return 1;
}

=item @array getListOfStocks()

  required args:
  optional: marketOnly => 0|1
  returns: Array of Stocks
  summary:  This routine selects stock objects.
    marketOnly - defaults to 0 to select all stocks
       if 1 then stocks with a market set to NONE will be ignored

=cut
sub getListOfStocks
{
  my $self = shift;
  my %args = ( marketOnly => 0, @_ );
  my $marketOnly = 0;
  my @stocks = ();
  my $sth;
  my @info;

  # validate input
  if ($marketOnly)
  {
    $self->{LIST}[0] = <<"FOO";
  SELECT
    name||' ('||ticker||')' AS name,
    sub_acct AS id
   FROM stocks_tb
   WHERE market <> 'None'
  ORDER BY name
FOO
  }

  if ($self->debug) { $self->debug("<p>".$self->{LIST}[0]."</p>\n"); }
  return $self->selectArrayHash("array");
}

# getStockMarkets
# requires:
# optional:
# returns: array of markets
# summary: gathers all unique markets from the stocks_tb
sub getStockMarkets
{
  my $self = shift;
  my @markets = ();
  my $sth;
  my @info;

  $sth = $self->{dbHandle}->read(sql => "SELECT DISTINCT(market) FROM stocks_tb WHERE market <> 'None' ORDER BY market");
  if ($self->{dbHandle}->error)
  {
    $self->prefixError(errorString => $self->{dbHandle}->errorMessage);
    return @markets;
  }
  while (@info = $sth->fetchrow_array)
  {
    $markets[++$#markets] = $info[0];
  }

  push @markets, "None";  #there may not be a NONe the first time this is run.  create it

  return @markets;
}

sub selectIdByTicker
{
  my $self = shift;
  my $ticker = shift;
  my $sql = <<"FOO";
  SELECT
    s.sub_acct AS id,
    s.name AS company,
    a.sub_acc_name AS name
  FROM stocks_tb AS s, sub_acc_tb AS a
  WHERE
    s.ticker = ?
    AND a.sub_acc_number = s.sub_acct
FOO
  my $result = $self->{dbHandle}->read(sql => $sql, plug => [ $ticker] );
  if ($self->{dbHandle}->error) { $self->error($self->{dbHandle}->errorMessage); }
  my $count = 0;
  my $err;
  while (my $d = $result->fetchrow_hashref)
  {
    $count++;
    $err .= "<p>$d->{name}($d->{company}) has ticker '$ticker'</p>\n";
    $self->set('id', $d->{id});
  }
  if ($count < 1)
  {
    $self->error("No stocks found with ticker: $ticker</p>\n");
    return 0;
  }
  elsif ($count > 1)
  {
    $self->error("More than one stock was found with ticker: $ticker<br>This usually happens because a stock is delisted and it's ticker is re-used.  Try replacing the ticker with an empty field on the stock.</p>\n");
    $self->error($err);
    return 0;
  }
  return 1;
}

sub delete
{
  my $self = shift;
  my $p = $self->langObj;
  my $watchlist = "SELECT '".$self->langObj->map('Watchlists')."' FROM stock_watch_tb WHERE sub_acct = ".$self->get('id');
  my $lot_node = "SELECT ".$self->langObj->map('Investments')." FROM lot_node_tb WHERE sub_account_num = ".$self->get('id');
  my $result = $self->{dbHandle}->read($watchlist);
  if ($self->{dbHandle}->error) { $self->invalid('database', $watchlist, $self->{dbHandle}->errorMessage); return -1; }
  if (my @data = $result->fetchrow_array)
  {
    $self->invalid($p->map('Sub Account'), $self->get('name'), $self->langObj->map('in use').":".$data[0] );
    return -1;
  }
  $result = $self->{dbHandle}->read($lot_node);
  if ($self->{dbHandle}->error) { $self->invalid('database', $lot_node, $self->{dbHandle}->errorMessage); return -1; }
  if (my @data = $result->fetchrow_array)
  {
    $self->invalid($p->map('Sub Account'), $self->get('name'), $self->langObj->map('in use').":".$data[0] );
    return -1;
  }
  return $self->SUPER::delete(@_);
}

=back

=cut

1;
__END__

=head1 NOTE

 All data fields are accessible by specifying the object
 and pointing to the data member to be modified on the
 left-hand side of the assignment.
 Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

Xperience, Inc. (mailto:admin@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal(3), Portal::Accounting(3), Portal::Accounting::Objects::Base(3)

=cut
