# JournalEntry.pm - The Object Class that provides a JournalEntry Object
#$Id: JournalEntry.pm,v 1.61.2.1 2006/09/08 22:18:28 moreejt Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.
# Created by Xperience, Inc.

package Portal::Accounting::JournalEntry;
use strict;
use Portal::Accounting::Objects::Account;
use Portal::Accounting::Objects::Search;
use Portal::Accounting::BaseL;
use Portal::Accounting::Objects::Ledger;
use JSON;
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

use Date::Calc qw(Today);

require Exporter;

@ISA = qw(Portal::Accounting::BaseL Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '$Revision: 1.61.2.1 $';

# global variables.

# modify to represent the commands this state provides.
my %commands = (
  check => "Check",
  creditPurchase => "Credit Purchase",
  creditReceipt => "Credit Receipt",
  delete => "Delete",
  demo => "Demo Data",
  deposit => "Deposit",
  display => "General Ledger",
  doubleEntry => "Double Entry",
  investment => "Investment",
  federalDeposit => "Check",
  federalDepositForm => "Federal 941 Deposit",
  edit => "Edit",
  insert => "Insert",
  insertAJAX => "Insert",
  opening => "Opening Balances",
  searchForm => "Search Journals",
  search => "Search Journals",
  select => "Recall",
  transfer => "Transfer",
  view => 'View',
);

=head1 NAME

JournalEntry

=head1 SYNOPSIS

  use Portal::Accounting::JournalEntry;
  my $obj = Portal::Accounting::JournalEntry->new;
  if ($obj->error())
  {
    die $obj->errorMessage();
  }
  $obj->run(command => $command);

=head1 DESCRIPTION

This set of screens will hold the forms for data Entry in the system.

=head1 Exported FUNCTIONS

B<NOTE>: I<bool> = 1(true), 0(false)

=over 4

=item scalar new(codeVersion, release, DBVersion, #!!!portal args??)

 Creates a new instance of the Portal::Accounting::JournalEntry module.
 See Portal::Accounting::Base(3) for a listing of required arguments.

 Instantiates an instance of the JournalEntry State. !!Anything passed in here should be
 data that never changes from one run of a command to another such as database handles
 portalSession? input? cookieObj? dont fit that criterion

=cut

sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);

  $self->{commands} = \%commands;

  if ($self->error)
  {
    $self->prefixError();
    return $self;
  }

  # instantiate anything unique to this module

  # do validation
  if (!$self->Portal::Accounting::JournalEntry::isValid)
  {
    # the error is set in the isValid() method.
    return $self;
  }

  # do anything else you might need to do.
  $self->{Journals} = {
    G => "doubleEntry",
    CP => "check",
    CSR => "deposit",
    CDP => "creditPurchase",
    CDR => "creditReceipt",
    TX => "transfer",
    IS => "investment",
    O => "opening",
  };
  return $self;
}

=item bool isValid(void)

 Returns 0 or 1 to indicate if the object is valid.
 The error will be available via errorMessage().

=cut

sub isValid
{
  my $self = shift;

  # make sure our Parent class is valid.
  if (!$self->SUPER::isValid())
  {
    $self->prefixError();
    return 0;
  }

  # validate our parameters.

  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->postfixError($self->genErrorString("all"));
    return 0;
  }

  return 1;
}

=item HTMLObject display()

    Implements the default screen for this state.

=cut

sub c_display
{
  my $self = shift;
  my %args = ( command => "display", message => "", @_ );
  my $command = $args{command};
  my $siteDir = $self->{configObj}->{siteDir};
  my $phrases = $self->{langObj};
  my $message = $args{message};
  my $ledger = $self->{input}->{ledger};
  my $ledgerCode = $self->{input}->{ledgerCode};

  my $doc = $self->prepDoc;
  my $url = $self->{baseURL};

  my $demo = $self->langObj->map("Demo Data");
  my $Check = $self->langObj->map("Check");
  my $Deposit = $self->langObj->map("Deposit");
  my $Double = $self->langObj->map("Double Entry");
  my $Credit = $self->langObj->map("Credit Purchase");
  my $CreditR = $self->langObj->map("Credit Receipt");
  my $Search = $self->langObj->map("Search");

  my $s = $self->{input}->{state};
  my $a = $self->{input}->{app};
  my $l = $self->{input}->{ledgerCode};
  $doc->print(<<"EOFO");
<ul>
  <li><a href="?app=$a&state=$s&command=check&ledgerCode=$l">$Check</a></li>
  <li><a href="?app=$a&state=$s&command=deposit&ledgerCode=$l">$Deposit</a></li>
  <li><a href="?app=$a&state=$s&command=doubleEntry&ledgerCode=$l">$Double</a></li>
  <li><a href="?app=$a&state=$s&command=creditPurchase&ledgerCode=$l">$Credit</a></li>
  <li><a href="?app=$a&state=$s&command=creditReceipt&ledgerCode=$l">$CreditR</a></li>
EOFO
  $self->{OBJECT} = Portal::Accounting::Objects::Search->new(langObj => $self->{langObj}, dbHandle => $self->{dbHandle}, ledgerCode => $self->{input}->{ledgerCode}, @_);
  $doc->print('  <li><a href="?app=Accounting&state=JournalEntry&command=searchForm&ledgerCode='.$ledgerCode.'" onclick="document.getElementById(\'search\').style.display = \'block\'; return false;">'.$Search.'</a></li>');

  if ($self->demoCheck)
  {
    $doc->print(<<"EOFO");
  <li><a href="?app=$a&state=$s&command=demo&ledgerCode=$l">$demo</a></li>
  <li><a href="?app=$a&state=$s&command=insertAJAX&ledgerCode=$l&date=2007-01-01&payee=ajaxtest&rows=2&account_1=10000&dorc_1=true&amount_1=10.00&account_0=10000&dorc_0=false&amount_0=10.00">AJAX test</a></li>
EOFO
  }

  $doc->print("\n</ul>\n");
  $doc->print('<div id="search" style="clear: both; display: none;">');
  $doc->print($self->{OBJECT}->printForm);
  $doc->print("</div>\n");
  return $doc;
}

sub c_doubleEntry
{
  my $self = shift;
  my %args = ( message => "", id => -1, @_ );
  my $doc = $self->prepDoc();
  use Portal::Accounting::Objects::JournalEntry::DoubleEntry;
  my $je = Portal::Accounting::Objects::JournalEntry::DoubleEntry->new(langObj => $self->{langObj}, ledgerCode => $self->{input}->{ledgerCode}, dbHandle => $self->dbHandle);
  if ($je->error)
  {
    $doc->print("<h4>Error Occurred while initializing Journal Entry Object</h4>".$je->errorMessage."<br>\n");
    $doc->print($je->errorMessage);
  }
  $je->set('id', $args{id});
  if ($je->error)
  {
    $doc->print("<h4>Error Occurred setting id field Journal Entry Object</h4>".$je->errorMessage."<br>\n");
    $doc->print($je->errorMessage);
  }
  else
  {
    $doc->setFocus("body");
    my %form = $je->printForm(form => "main", command => "display");
    if ($je->error)
    {
      $doc->print("Error<br>".$je->errorMessage);
    }
    $doc->print(%form);
  }
  $doc->setFocus('body');
  $doc->print($self->accountMap);
  return $doc;
}

sub c_opening
{
  my $self = shift;
  my %args = ( message => "", @_ );
  my $doc = $self->prepDoc();
  use Portal::Accounting::Objects::JournalEntry::Opening;
  my $je = Portal::Accounting::Objects::JournalEntry::Opening->new(langObj => $self->{langObj}, ledgerCode => $self->{input}->{ledgerCode}, dbHandle => $self->dbHandle);
  if ($je->error)
  {
    $doc->print("<h4>Error Occurred while initializing Journal Entry Object</h4>".$je->errorMessage."<br>\n");
    $doc->print($je->errorMessage);
  }
  my $opening = $je->openingJournalEntry();
  if ($je->error) { $doc->print($je->errorMessage); }
  else
  {
    if ($opening eq '') {$opening = -1; }
  }
  $je->set('id', $opening);
  if ($je->error)
  {
    $doc->print("<h4>Error Occurred setting id field Journal Entry Object</h4>".$je->errorMessage."<br>\n");
    $doc->print($je->errorMessage);
  }
  else
  {
    $doc->setFocus("body");
    my %form = $je->printForm(form => "main", command => "display");
    if ($je->error)
    {
      $doc->print("Error<br>".$je->errorMessage);
    }
    $doc->print(%form);
  }

  $doc->setFocus('body');
  $doc->print($self->accountMap);
  return $doc;
}

sub c_federalDepositForm { my $self = shift; return $self->c_federalDeposit(@_); }
sub c_federalDeposit
{
  use Portal::Accounting::Objects::JournalEntry::Check;
  use Portal::Accounting::Objects::Payroll::Fed941Deposit;
  my $self = shift;
  my %args = ( message => "", @_ );
  my $doc = $self->prepDoc();
  my $fed = Portal::Accounting::Objects::Payroll::Fed941Deposit->new(ledgerCode => $self->{input}->{ledgerCode}, dbHandle => $self->{dbHandle}, langObj => $self->{langObj});
  if ($self->{input}->{command} eq 'federalDeposit')
  {
    if ($fed->populate(%{$self->{input}}))
    {
      $fed->select;
      if ($fed->error) { $doc->print($fed->errorMessage); return $doc; }
      if ($fed->get('total') == 0)
      {
        $doc->print("<p>".$self->{langObj}->map('NoFedData')." :".$fed->get('year')."/".$fed->get('month')."</p>\n");
      }
      else
      {
        $doc->print($fed->javascript);
        my $lo = Portal::Accounting::Objects::Ledger->new(langObj => $self->{langObj}, dbHandle => $self->dbHandle);
        $lo->set('code', $self->{input}->{ledgerCode});
        $lo->select();
        if ($lo->error) { $self->error("<p>Could not create ledger!</p>\n".$lo->errorMessage); return undef; }
        my $je = Portal::Accounting::Objects::JournalEntry::Check->new(langObj => $self->{langObj}, ledgerCode => $self->{input}->{ledgerCode}, dbHandle => $self->dbHandle, ledger => $lo);
        if ($je->error)
        {
          $doc->print("<h4>Error Occurred while initializing Journal Entry Object</h4>".$je->errorMessage."<br>\n");
          $doc->print($je->errorMessage);
        }
        $je->set('id', -2);
        if ($je->error)
        {
          $doc->print("<h4>Error Occurred setting id field Journal Entry Object</h4>".$je->errorMessage."<br>\n");
          $doc->print($je->errorMessage);
        }
        else
        {
          $doc->setFocus("body");
          my %form = $je->printForm(form => "main", command => "display");
          if ($je->error)
          {
            $doc->print("Error<br>".$je->errorMessage);
          }
          $doc->print(%form);
        }
        return $doc;
      }
    }
    else { $doc->print('pop fAILED!'); }
  }
  #display form
  $doc->print($fed->printForm);
# $doc->print($fed->{LIST}[0]);
# $doc->print($fed->{debugString});
  return $doc;
}

sub c_check
{
  use Portal::Accounting::Objects::JournalEntry::Check;
  my $self = shift;
  my %args = ( message => "", id => -1, @_ );
  if (defined $self->{input}->{id} && $args{id} == -1) { $args{id} = $self->{input}->{id}; }
  my $doc = $self->prepDoc();
  my $je = Portal::Accounting::Objects::JournalEntry::Check->new(langObj => $self->{langObj}, ledgerCode => $self->{input}->{ledgerCode}, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});
  if ($je->error)
  {
    $doc->print("<h4>Error Occurred while initializing Journal Entry Object</h4>".$je->errorMessage."<br>\n");
    $doc->print($je->errorMessage);
  }
  $je->set('id', $args{id});
  if ($je->error)
  {
    $doc->print("<h4>Error Occurred setting id field Journal Entry Object</h4>".$je->errorMessage."<br>\n");
    $doc->print($je->errorMessage);
  }
  else
  {
    $doc->setFocus("body");
    my %form = $je->printForm(form => "main", command => "display");
    if ($je->error)
    {
      $doc->print("Error<br>".$je->errorMessage);
    }
    $doc->print(%form);
  }

  $doc->setFocus('body');
  $doc->print($self->accountMap);
  return $doc;
}

sub c_creditPurchase
{
  use Portal::Accounting::Objects::JournalEntry::CreditPurchase;
  my $self = shift;
  my %args = ( message => "", id => -1, @_ );
  if (defined $self->{input}->{id} && $args{id} == -1) { $args{id} = $self->{input}->{id}; }
  my $doc = $self->prepDoc();
  my $je = Portal::Accounting::Objects::JournalEntry::CreditPurchase->new(langObj => $self->{langObj}, ledgerCode => $self->{input}->{ledgerCode}, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});
  if ($je->error)
  {
    $doc->print("<h4>Error Occurred while initializing Journal Entry Object</h4>".$je->errorMessage."<br>\n");
    $doc->print($je->errorMessage);
  }
  $je->set('id', $args{id});
  if ($je->error)
  {
    $doc->print("<h4>Error Occurred setting id field Journal Entry Object</h4>".$je->errorMessage."<br>\n");
    $doc->print($je->errorMessage);
  }
  else
  {
    $doc->setFocus("body");
    my %form = $je->printForm(form => "main", command => "display");
    if ($je->error)
    {
      $doc->print("Error<br>".$je->errorMessage);
    }
    $doc->print(%form);
  }

  $doc->setFocus('body');
  $doc->print($self->accountMap);
  return $doc;
}

sub c_deposit
{
  use Portal::Accounting::Objects::JournalEntry::Deposit;
  my $self = shift;
  my %args = ( message => "", id => -1, @_ );
  if (defined $self->{input}->{id} && $args{id} == -1) { $args{id} = $self->{input}->{id}; }
  my $doc = $self->prepDoc();
  my $je = Portal::Accounting::Objects::JournalEntry::Deposit->new(langObj => $self->{langObj}, ledgerCode => $self->{input}->{ledgerCode}, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});
  if ($je->error)
  {
    $doc->print("<h4>Error Occurred while initializing Journal Entry Object</h4>".$je->errorMessage."<br>\n");
    $doc->print($je->errorMessage);
  }
  $je->set('id', $args{id});
  if ($je->error)
  {
    $doc->print("<h4>Error Occurred setting id field Journal Entry Object</h4>".$je->errorMessage."<br>\n");
    $doc->print($je->errorMessage);
  }
  else
  {
    $doc->setFocus("body");
    my %form = $je->printForm(form => "main", command => "display");
    if ($je->error)
    {
      $doc->print("Error<br>".$je->errorMessage);
    }
    $doc->print(%form);
  }

  $doc->setFocus('body');
  $doc->print($self->accountMap);
  return $doc;
}

sub c_creditReceipt
{
  use Portal::Accounting::Objects::JournalEntry::CreditReceipt;
  my $self = shift;
  my %args = ( message => "", id => -1, @_ );
  if (defined $self->{input}->{id} && $args{id} == -1) { $args{id} = $self->{input}->{id}; }
  my $doc = $self->prepDoc();
  my $je = Portal::Accounting::Objects::JournalEntry::CreditReceipt->new(langObj => $self->{langObj}, ledgerCode => $self->{input}->{ledgerCode}, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});
  if ($je->error)
  {
    $doc->print("<h4>Error Occurred while initializing Journal Entry Object</h4>".$je->errorMessage."<br>\n");
    $doc->print($je->errorMessage);
  }
  $je->set('id', $args{id});
  if ($je->error)
  {
    $doc->print("<h4>Error Occurred setting id field Journal Entry Object</h4>".$je->errorMessage."<br>\n");
    $doc->print($je->errorMessage);
  }
  else
  {
    $doc->setFocus("body");
    my %form = $je->printForm(form => "main", command => "display");
    if ($je->error)
    {
      $doc->print("Error<br>".$je->errorMessage);
    }
    $doc->print(%form);
  }

  $doc->setFocus('body');
  $doc->print($self->accountMap);
  return $doc;
}

sub c_investment
{
  my $self = shift;
  my %args = (id => -1, @_);
  my $doc = $self->prepDoc;
  #find investment transaction that goes with this journal entry
  #display it on screen with links to helpful stuff?
  use Portal::Accounting::Investments::Objects::Event;
  my $event = Portal::Accounting::Investments::Objects::Event->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});
  use Portal::Accounting::Investments::Objects::Search;
  my $search = Portal::Accounting::Investments::Objects::Search->new(langObj => $self->{langObj}, dbHandle => $self->{dbHandle}, ledgerCode => $self->{input}->{ledgerCode}, @_);
  if (!defined $search) { $doc->print('CREATE SEARCH FAILED'); return $doc; }
  if ($search->error) { $doc->print($search->errorMessage); return $doc; }
  my $sql = "SELECT event_ref AS id FROM lot_node_tb WHERE journal_ref = ?";
  my @plug = ($args{id});
  my $sth = $self->dbHandle->read(sql => $sql, plug => \@plug);
  if ($self->dbHandle->error) { $self->error($self->dbHandle->errorMessage); }
  else
  {
    if (my @d = $sth->fetchrow_array)
    {
      my $e = $event->getEvent($d[0]);
      if ($event->error) { $self->error($event->errorMessage); }
      elsif ($e->error) { $self->error($e->errorMessage); }
      else
      {
        my $stock = $e->get('stock');
        my $account = $e->get('account');
        $search->populate(stocks => $stock, accounts => $account );
        if ($search->error) { $doc->print($search->errorMessage); return $doc; }
        $doc->print('<b>'.$self->langObj->map('Account').'</b>: '.$search->lookupSelectName(item => 'accounts'));
        $doc->print(' - <b>'.$self->langObj->map('Stock').'</b>: '.$search->lookupSelectName(item => 'stocks'));
        $doc->print($e->printHeaders);
        $doc->print($e->printReport);
        $doc->print($search->printForm);
      }
    }
    else { $self->error('<div class="error">Should not have gotten here.  Found no data in lot_node_tb for journal_ref = '.$self->{input}->{id}."</div>"); }
  }
  return $doc;
}

#select is to retrieve a javascript version of a journal entry to be used by the data entry screen
sub c_select
{
  my $self = shift;
  my $doc = HTMLObject::Normal->new();
  my $id = $self->{input}->{id};
  if (defined $self->{input}->{bank} && defined $self->{input}->{dorc})
  {
    #find the correct entry by using search
    my $sql = "SELECT parent_num AS id FROM debitcredit_tb WHERE ledger_code =? AND account_num = ? AND dorc = ? AND sub_account_num = ? ORDER BY id DESC LIMIT 1";
    my @plug = ($self->{input}->{ledgerCode}, $self->{input}->{bank}, $self->{input}->{dorc}, $self->{input}->{id});
    my $sth = $self->dbHandle->read(sql => $sql, plug => \@plug);
    if ($self->dbHandle->error) { return $self->ajaxError($self->dbHandle->errorMessage); }
    if (my @data = $sth->fetchrow_array) { $id = $data[0]; }
    else { return $self->ajaxError('No transaction found for '.$sql.join(",",@plug)); }
  }
  $doc->setContentType('text/plain');
  my $j = Portal::Accounting::Objects::JournalEntry->new(langObj => $self->{langObj}, dbHandle => $self->{dbHandle}, ledgerCode => $self->{input}->{ledgerCode});
  if (! defined $j) { return $self->ajaxError("<p>Could not create JournalEntry Object</p>\n"); }
  if ($j->error) { return $self->ajaxError("<p>Could not use object: ".$j->errorMessage."</p>\n"); }
  $j->set('id', $id);
  if ($j->select() == 0)
  {
    $j->set($j->{id}, -1);
  }
  if ($j->error) { return $self->ajaxError($j->errorMessage); }
  else
  {
    $doc->print($j->json());
  }
  return $doc;
}

=item HTMLObject c_view

  This selects the journalentry based on the given ID and output the General Ledger view of it.

=cut
sub c_view
{
  my $self = shift;
  my $doc = $self->prepDoc;
  my $j = Portal::Accounting::Objects::JournalEntry->new(langObj => $self->{langObj}, dbHandle => $self->{dbHandle}, ledgerCode => $self->{input}->{ledgerCode});
  if ($j->error) { $self->error('<div class="error">Could not use object: '.$j->errorMessage."</div>\n"); return undef; }
  if (!defined $self->{input}->{id}) { $self->error('<div class="error">id is required.  No id sent to JournalEntry->view</div>'."\n"); return undef; }
  $j->set('id', $self->{input}->{id});
  if ($j->select)
  {
    $doc->print($j->printHeaders);
    if ($j->error) { $doc->print($j->errorMessage); }
    else { $doc->print($j->printForm); }
  }
  return $doc;
}

sub ajaxError
{
  my $self = shift;
  my %obj;
  $obj{ERROR} = shift;
  $obj{STATUS} = -1;
  my $doc = HTMLObject::Normal->new;
  $doc->setContentType("text/plain");
  $doc->print("var obj  = (".objToJson(\%obj).");");
  return $doc;
}

sub c_insert
{
  my $self = shift;
  my $doc = $self->prepDoc();
  my $lc = $self->{input}->{ledgerCode};
  $doc->setFocus("body");
  my $acc = Portal::Accounting::Objects::Account->new(langObj => $self->{langObj}, dbHandle => $self->{dbHandle});
  if ($acc->error)
  {
    $self->error("<p>Could not create Objects::Account in c_insert</p>\n");
    return undef;
  }
  my $success = '-1';
  my $j = Portal::Accounting::Objects::JournalEntry->new(langObj => $self->{langObj}, dbHandle => $self->{dbHandle}, ledgerCode => $lc);
  if ($j->error)
  {
    $doc->print('<div class="error">new failed</div>'.$j->errorMessage);
    return $doc;
  }
  $j->populate(%{$self->{input}});
  $doc->print($j->debugString);
  if ($j->error)
  {
    $doc->print('<div class="error">populate failed</div>'.$j->errorMessage);
  }
  else
  {
    $j->write(commit => 1);
    if ($j->error)
    {
      $doc->print('<div class="error">database write failed</div>'.$j->errorMessage);
    }
    else
    {
      $doc->print($j->printHTML);
      $success = $j->get("id");
    }
  }
  if ($j->warning) { $doc->print($j->warningString); }
  $doc->print($j->debugString);
  my $js = $acc->bankOverhead($lc);
  $doc->print($js);
  $doc->setFocus("javascript");
  $doc->print($js);
  $doc->print(<<"ENDOF");
  var js = '$js';
  if (opener)
  {
    if (opener.banks) opener.banks(js);
    else { alert("function banks not found"); }
    if (opener.postInsert) opener.postInsert($success);
    else { alert("function postInsert not found"); }
  }
ENDOF
  return $doc;
}

sub c_insertAJAX
{
  my $self = shift;
  use JSON;
  my $status = -1;
  my $error;
  my $doc = HTMLObject::Normal->new;
  $doc->setContentType('text/plain');
  my $acc = Portal::Accounting::Objects::Account->new(langObj => $self->{langObj}, dbHandle => $self->{dbHandle});
  if ($acc->error)
  {
    $self->error("<div>Could not create Objects::Account in c_insert</div>");
    return undef;
  }
  my $success = '-1';
  my $j = Portal::Accounting::Objects::JournalEntry->new(langObj => $self->{langObj}, dbHandle => $self->{dbHandle}, ledgerCode => $self->{input}->{ledgerCode});
  if ($j->error)
  {
    $error .= '<div class="error">new JournalEntry failed</div>'.$j->errorMessage;
  }
  $j->populate(%{$self->{input}});
  if ($j->error)
  {
    $error .= '<div class="error">populate failed</div>'.$j->errorMessage;
  }
  else
  {
    $j->write(commit => 1);
    if ($j->error || $j->dataError)
    {
      $error .= '<div class="error">database write failed</div>'.$j->errorMessage.$j->genErrorString('all');
    }
    else
    {
#      $doc->print($j->json);
      $status = $j->get("id");
    }
  }
  my %obj = ( STATUS => $status, ERROR => $error);
  $doc->print("var obj = (".objToJson(\%obj).");");
  $doc->print($acc->bankOverhead($self->{input}->{ledgerCode}) );
  return $doc;
}

sub c_demo
{
  use Portal::Accounting::Objects::Demo;
  my $self = shift;
  my %args = ( message => "", @_ );
  my $phrases = $self->{langObj};
  my $lc = $self->{input}->{ledgerCode};
  my $year = 2004;
  my $doc = $self->prepDoc();
  $doc->setFocus("body");
  if (!$self->demoCheck)
  {
    $doc->print('<p class="error">Demo is not available.  Either you are not an administrator or');
    $doc->print("demo is disabled!  You must set demo = 1 in the config_tb.</p>");
    return $doc;
  }
  my $DEMO = Portal::Accounting::Objects::Demo->new(langObj => $self->{langObj}, ledgerCode => $lc, dbHandle => $self->{dbHandle});
  if ($DEMO->error) {$doc->print($DEMO->errorMessage);}
  if ($self->{input}->{confirmDemo} ne "1")
  {
    $doc->print($DEMO->printForm);
    if ($DEMO->error) {$doc->print($DEMO->errorMessage);}
    return $doc;
  }
  if (!$DEMO->populate(%{$self->{input}}) )
  {
    $doc->print($DEMO->errorMessage);
    if ($DEMO->error) {$doc->print($DEMO->errorMessage);}
    return $doc;
  }
  $doc = $DEMO->printReport;
  $doc->print(<<"ENDOF");
<form name="main">
  <input type="hidden" name="command" value="edit" />
  <input type="hidden" name="app" value="$self->{app}" />
  <input type="hidden" name="state" value="JournalEntry" />
  <input type="hidden" name="ledgerCode" value="$self->{input}->{ledgerCode}" />
  Enter an id to edit a transaction:
  <input type="text" name="id" value="1" />
  <input type="submit">
</form>
ENDOF
  if ($DEMO->error) {$doc->print($DEMO->errorMessage);}
  return $doc;
}

sub c_edit
{
  my $self = shift;
  my $doc = $self->prepDoc;
  my $sql = "SELECT journal AS journal FROM journal_entry_tb WHERE ref_num = '$self->{input}->{id}'";
  #get Journal Code for given id and then create object of correct type or call correct subroutien
  my $sth = $self->{dbHandle}->read($sql);
  my $name = "doubleEntry";
  if (my @code = $sth->fetchrow_array)
  {
    $name = $self->{Journals}->{$code[0]};
  }
  else { $self->error("Could not get journal in JournalEntry->c_edit:$sql"); }
  my $r;
  eval('$r = $self->c_'.$name.'(id => $self->{input}->{id}, @_);');
  return $r;
}

sub c_delete
{
  my $self = shift;
  use JSON;
  my $status = -1;
  my $error;
  my $doc = HTMLObject::Normal->new;
  $doc->setContentType('text/plain');
  if ($self->{input}->{id} !~ /^\d+$/)
  {
    $error ="<p>Invalid transaction id sent to delete: $self->{input}->{id}</p>\n";
  }
  else
  {
    #create new journal entry
    my $l = Portal::Accounting::Objects::Ledger->new(langObj => $self->{langObj}, dbHandle => $self->{dbHandle});
    $l->set('code', $self->{input}->{ledgerCode});
    $l->select;
    if ($l->error) { $error .= "<p>Could not retrieve ledger object from database for code ".$self->{input}->{ledgerCode}."!</p>\n"; }
    else
    {
      my $o = Portal::Accounting::Objects::JournalEntry->new(langObj => $self->{langObj}, dbHandle => $self->{dbHandle}, ledger => $l, @_);
      if ($o->error) { $error .= "<p>Could not use object: ".$o->errorMessage."</p>\n";  }
      else
      {
        $o->set('id', $self->{input}->{id});
        if (! $o->error)  {  $status = $o->delete;  }
        if ($o->error) { $error = $o->errorMessage; }
        else
        {
          if ($status == -1)
          {
            $error = '<div>Delete failed</div>'.$o->errorMessage;
          }
          else
          {
            $error = 'Successfully deleted ';
            $status = $o->get('id');
          }
        }
      }
    }
  }
  my %obj = ( STATUS => $status, ERROR => $error);
  $doc->print("var obj = (".objToJson(\%obj).");");

  return $doc;
}

sub c_searchForm
{
  my $self = shift;
  $self->{OBJECT} = Portal::Accounting::Objects::Search->new(langObj => $self->{langObj}, dbHandle => $self->{dbHandle}, ledgerCode => $self->{input}->{ledgerCode}, @_);
  return $self->SUPER::c_searchForm(@_);
}

sub prepDoc
{
  my $self = shift;
  my $doc = $self->SUPER::prepDoc(@_);
  $doc->setJavascriptInclude(file => $self->{jsDir}."Journal.js");
  return $doc;
}

=back

=cut

1;
__END__

=head1 NOTE

 All data fields are accessible by specifying the object
 and pointing to the data member to be modified on the
 left-hand side of the assignment.
 Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

PC & Web Xperience, Inc. (mailto:xiwa@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal(3), Portal::Accounting(3), Portal::AppState(3)

=cut
